package
{
	import Box2D.Collision.*;
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.Math.*;
	import Box2D.Common.b2Settings;
	import Box2D.Dynamics.*;
	
	import flash.display.*;
	import flash.events.*;
	import flash.geom.*;
	import flash.net.*;
	import flash.utils.*;
	
	import org.aswing.*;
	import org.aswing.event.*;
	import org.aswing.geom.IntDimension;

	public class DrawObject extends Sprite
	{
		
		//Box2D
		private var m_world:b2World;
		private var m_physScale:Number = 10;
		
		//描画
		private var currentShape:Shape;
		private var vertexList:Array;
		
		//マウス
		private var isMouseDown:Boolean = false;
		
		//色
		private const KULER_URL:String = "http://kuler.adobe.com/kuler/services/theme/getList.cfm?listtype=popular";
		private var swatchList:Array = new Array();
		private var currentColor:uint;
		
		//オブジェクトタイプ
		private const T_BOX:String = "BOX";
		private const T_CIRCLE:String = "CIRCLE";
		private const T_POLY:String = "POLY";
		private var currentType:String = T_BOX;
		
		//Kulerのロゴ
		[Embed(source="ku_36pxWtext.png")]
		private var kulerLogo:Class;
		
		/* ====== Constructor ============================== */	
		
		public function DrawObject()
		{
			stage.scaleMode = "noScale";
			stage.align = "TL";
			AsWingManager.setRoot( this );
			
			//Call Kuler
			var loader:URLLoader = new URLLoader();
			loader.addEventListener(Event.COMPLETE, onKulerLoadComplete);
			var request:URLRequest = new URLRequest(KULER_URL);
			try {
				loader.load(request);
			} catch (error:Error) {
				trace("Unable to load requested document.");
			}
		}
		
		/* ====== Initialize ============================== */	
		
		private function init() :void
		{
			// Add Kuler Logo
			var logo:Bitmap = new kulerLogo();
			logo.x = stage.stageWidth - logo.width -20;
			logo.y = 3;
			addChild(logo);
			
			addEventListener(Event.ENTER_FRAME, update, false, 0, true);
			
			// Creat world AABB
			var worldAABB:b2AABB = new b2AABB();
			worldAABB.minVertex.Set(0, 0);
			worldAABB.maxVertex.Set(1600/m_physScale, 1600/m_physScale);
			
			// Define the gravity vector
			var gravity:b2Vec2 = new b2Vec2(0.0, 100.0);
			
			// Allow bodies to sleep
			var doSleep:Boolean = true;
			
			// Construct a world object
			m_world = new b2World(worldAABB, gravity, doSleep);
			
			// Create Ground
			createGround();
			
			// Add Mouse Listener
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onStageMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onStageMouseUp);
			stage.addEventListener(Event.MOUSE_LEAVE , onStageMouseUp);
			
			createAsWing();
		}
		
		
		
		/* ====== Listeners ============================== */	
		
		// call in each frame
		private function updateLine( e :Event ):void
		{	
			var x:Number = mouseX-currentShape.x;
			var y:Number = mouseY-currentShape.y;
			currentShape.graphics.lineTo(x, y);
			vertexList.push(new Point(x/m_physScale, y/m_physScale));
		}
		
		// call in each frame
		private function update(e:Event):void
		{
			m_world.Step(1 / 30, 10);
			
			 // Render
			graphics.clear();
			for (var bb:b2Body = m_world.m_bodyList; bb; bb = bb.m_next) {
				for (var s:b2Shape = bb.GetShapeList(); s != null; s = s.GetNext()) {
					drawShape(bb, s);
				}
			}
		}
		
		private function onStageMouseDown(e:MouseEvent):void
		{
			isMouseDown = true;
			vertexList = new Array();
			
			var canvas:Shape = new Shape();
			canvas.x = mouseX;
			canvas.y = mouseY;
			
			var color:uint = swatchList[Math.random()*swatchList.length>>0];
			while(color == 0xFFFFFF){
				color = swatchList[Math.random()*swatchList.length>>0];
			}
			
			with(canvas.graphics){
				lineStyle(1,color,1);
				moveTo(0, 0);
				beginFill(color, 0.9);
			}
			currentShape = canvas;
			currentColor = color;
			
			addChild(canvas);
			addEventListener(Event.ENTER_FRAME, updateLine);
		}
		
		private function onStageMouseUp(e:*):void
		{
			if(isMouseDown){
				removeEventListener(Event.ENTER_FRAME, updateLine);
				currentShape.graphics.endFill();
				createB2Shape(currentType);
				removeChild(currentShape);
				isMouseDown = false;
			}
		}
		
		private function onKulerLoadComplete(e:Event):void
		{
            
			var xml:XML = new XML(e.target.data);
			var allSwatchesCount:uint = xml..swatches.length();
			var targetSwatches:XML = xml..swatches[Math.random()*allSwatchesCount>>0];
			
			for each(var swatch:XML in targetSwatches.swatch){
				var r:uint = swatch.c1*0xFF >> 0;
				var g:uint = swatch.c2*0xFF >> 0;
				var b:uint = swatch.c2*0xFF >> 0;
				var color:uint = r << 16 | g << 8 | b;
				swatchList.push(color);
			}
			init();
		}
				
		private function onAsWingButtonChange(e:InteractiveEvent):void
		{
			currentType = e.target.getDisplayText();
		}
		
		
		/* ====== Operations ============================== */	
		
		public function drawShape(body:b2Body, shape:b2Shape):void {
			
			switch (shape.m_type)
			{
			case b2Shape.e_circleShape:
				{
					var circle:b2CircleShape = shape as b2CircleShape;
					var pos:b2Vec2 = circle.m_position;
					var r:Number = circle.m_radius;
					var k_segments:Number = 16.0;
					var k_increment:Number = 2.0 * Math.PI / k_segments;
					graphics.beginFill(body.GetUserData(), 0.9);
					graphics.lineStyle(1,0xffffff,1);
					graphics.moveTo((pos.x + r) * m_physScale, (pos.y) * m_physScale);
					var theta:Number = 0.0;
					
					for (var i:int = 0; i < k_segments; ++i)
					{
						var d:b2Vec2 = new b2Vec2(r * Math.cos(theta), r * Math.sin(theta));
						var v:b2Vec2 = b2Math.AddVV(pos , d);
						graphics.lineTo((v.x) * m_physScale, (v.y) * m_physScale);
						theta += k_increment;
					}
					graphics.lineTo((pos.x + r) * m_physScale, (pos.y) * m_physScale);
					
					graphics.moveTo((pos.x) * m_physScale, (pos.y) * m_physScale);
					var ax:b2Vec2 = circle.m_R.col1;
					var pos2:b2Vec2 = new b2Vec2(pos.x + r * ax.x, pos.y + r * ax.y);
					graphics.lineTo((pos2.x) * m_physScale, (pos2.y) * m_physScale);
					graphics.endFill();
				}
				break;
			case b2Shape.e_polyShape:
				{
					var poly:b2PolyShape = shape as b2PolyShape;
					var tV:b2Vec2 = b2Math.AddVV(poly.m_position, b2Math.b2MulMV(poly.m_R, poly.m_vertices[i]));
					graphics.beginFill(body.GetUserData(), 0.9);
					graphics.lineStyle(1,0x000000,0);
					graphics.moveTo(tV.x * m_physScale, tV.y * m_physScale);
					for (i = 0; i < poly.m_vertexCount; ++i) {
						v = b2Math.AddVV(poly.m_position, b2Math.b2MulMV(poly.m_R, poly.m_vertices[i]));
						graphics.lineTo(v.x * m_physScale, v.y * m_physScale);
						
					}
					graphics.lineTo(tV.x * m_physScale, tV.y * m_physScale);
					graphics.endFill();
					
				}
				break;
			}
		}
		
		private function createB2Shape(type:String):void
		{
			
			var rect:Rectangle = currentShape.getRect(this);
			
			// Add sprite to body userData
			var shapeBody:b2BodyDef = new b2BodyDef();
			shapeBody.userData = currentColor;
			
			switch(type){
				case T_BOX:
				{
					shapeBody.position.x = rect.x/m_physScale+rect.width/m_physScale/2;
					shapeBody.position.y = rect.y/m_physScale+rect.height/m_physScale/2;
					
					var shapeBox:b2BoxDef = new b2BoxDef();
					shapeBox.extents.Set(rect.width/m_physScale/2, rect.height/m_physScale/2);
					shapeBox.density = 1.0;
					shapeBox.friction = 0.5;
					shapeBox.restitution = 0.2;
					shapeBody.AddShape(shapeBox);
				}
				break;
				case T_CIRCLE:
				{
					shapeBody.position.x = rect.x/m_physScale+rect.width/m_physScale/2;
					shapeBody.position.y = rect.y/m_physScale+rect.height/m_physScale/2;
					
					var shapeCircle:b2CircleDef = new b2CircleDef();
					shapeCircle.radius = Math.max(rect.width/m_physScale/2, rect.height/m_physScale/2);
					shapeCircle.density = 1.0;
					shapeCircle.friction = 0.5;
					shapeCircle.restitution = 0.2;
					shapeBody.AddShape(shapeCircle);
				}
				break;
				case T_POLY:
				{
					shapeBody.position.x = rect.x/m_physScale+(currentShape.x-rect.x)/m_physScale;
					shapeBody.position.y = rect.y/m_physScale+(currentShape.y-rect.y)/m_physScale;
					
					var shapePoly:b2PolyDef = new b2PolyDef();
					shapePoly.density = 1.0;
					shapePoly.friction = 0.5;
					shapePoly.restitution = 0.2;
					
					var convex:Array;
					if(vertexList.length>1){
						convex = ConvexHull.getConvexHull(vertexList);
					}else{
						convex = vertexList;
					}
					
					var maxCount:int = Math.min(convex.length, b2Settings.b2_maxPolyVertices);
					var gridCount:Number = convex.length/maxCount;
					var thinVertices:Array = new Array();
					
					for(var i:uint =0;i<maxCount;i++){
						var count:uint = i*gridCount>>0;
						thinVertices.push(convex[count]);
					}
					
					for(i=0;i<thinVertices.length;i++){
						shapePoly.vertices[i] = new b2Vec2(thinVertices[i].x, thinVertices[i].y);
					}
					
					shapePoly.vertexCount = thinVertices.length;
					
					shapeBody.AddShape(shapePoly);
				}
				break;
			}
			
			m_world.CreateBody(shapeBody);
		}
		
		private function createGround():void
		{
			//Create ground Shape
			var w:uint = stage.stageWidth;
			var h:uint = stage.stageHeight;
			
			var color:uint = swatchList[Math.random()*swatchList.length>>0];
			while(color == 0xFFFFFF){
				color = swatchList[Math.random()*swatchList.length>>0];
			}
			
			var floorBox:b2BoxDef = new b2BoxDef();
			floorBox.extents.Set(w/m_physScale/2, h/m_physScale/2);
			
			// Add ground body
			var floorBody:b2BodyDef = new b2BodyDef();
			floorBody.position.Set(w/m_physScale/2, (h-20+h/2)/m_physScale);
			floorBody.userData = color;
			
			floorBody.AddShape(floorBox);
			m_world.CreateBody(floorBody);
		}
		
		private function createAsWing() : JFrame{
			var frame : JFrame = new JFrame( this, "type" );
			frame.setComBoundsXYWH(-5, -32, 180, 70);
			
			var panel:JPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
			var group:ButtonGroup = new ButtonGroup();
			
			var button1:JRadioButton = new JRadioButton(T_BOX);
			button1.setSelected(true);
			group.append(button1);
			panel.append(button1);
			button1.addEventListener(InteractiveEvent.SELECTION_CHANGED, onAsWingButtonChange);
			
			var button2:JRadioButton = new JRadioButton(T_CIRCLE);
			group.append(button2);
			panel.append(button2);
			button2.addEventListener(InteractiveEvent.SELECTION_CHANGED, onAsWingButtonChange);
			
			var button3:JRadioButton = new JRadioButton(T_POLY);
			group.append(button3);
			panel.append(button3);
			button3.addEventListener(InteractiveEvent.SELECTION_CHANGED, onAsWingButtonChange);
	        
			frame.setContentPane( panel );
	        frame.setClipSize(new IntDimension(165, 55));
			frame.show();
			return frame;
		}
		
	}
}

//凸面の殻
import flash.geom.*;
class ConvexHull{
	
    public static function getConvexHull(points:Array):Array {

        var result:Array = [];
        var index:int;
        var topIndex:int;
		
        // 先頭のインデックスを求める
        var p1:Point = new Point(-100,-100);
        var p2:Point = new Point(0,-100);
        topIndex = index = getNextIndex(points, p2, p1, -1);

        var cnt:int = 0;
        while(cnt++ <= points.length){
            p1 = p2;
            p2 = points[index];
            result.push(p2);
            
           
            index = getNextIndex(points, p2, p1, index);
            if(index == topIndex)
                break;
        }

        return result;
    }

    private static function distance( p1:Point, p2:Point ):Number{
        var dx:Number = p2.x - p1.x;
        var dy:Number = p2.y - p1.y;
        return Math.sqrt( dx*dx+ dy*dy );
    }

    private static function getNextIndex(points:Array, p2:Point, p1:Point, myself:int):int{
        var minIndex:int = 0;
        var minLen:Number = -1;
        var min:Number = Math.PI*2;
        var v1:Point = new Point(p2.x - p1.x, p2.y - p1.y);

        for( var j:int =0; j < points.length; j++ ) {
            if(j != myself){
                var v2:Point = new Point(points[j].x - p2.x, points[j].y - p2.y);

                var rad:Number = getTheta(v1, v2);
                if(rad == -1){continue;}
                
                // 角度が同じ場合、近いほうを優先
                if(rad == min){
                    if(distance(points[j], p2) < minLen){
                        minIndex = j;
                        minLen = distance(points[j], p2);
                    }
                }else if(rad < min){
                    minIndex = j;
                    min = rad;
                    minLen = distance(points[j], p2);
                }
            }
        }
        return minIndex;
    }

    private static function getTheta(p1:Point, p2:Point):Number{
        var len1:Number =  Math.sqrt(p1.x*p1.x + p1.y*p1.y);
        var len2:Number =  Math.sqrt(p2.x*p2.x + p2.y*p2.y);
        if(len1 == 0 || len2==0){
            return -1;
        }
        p1.x = p1.x/len1;
        p1.y = p1.y/len1;
        p2.x = p2.x/len2;
        p2.y = p2.y/len2;

        var dot:Number = p1.x*p2.x + p1.y*p2.y;
        var theta:Number = Math.acos(dot);
        return theta;
    }
}

